#Requires -Version 5.0
#Requires -RunAsAdministrator

# Script to patch Compass Server Systems to the Log4jShell exploit
#
# The script applies two patches:
# 	1. disable the lookup of message patterns (works only for log4j version above 2.10
#	2. remove the JndiLookup class of log4j2

#---------------------------------------------------------------------------------
# Start logging to file
$timestamp = Get-Date -Format o | ForEach-Object { $_ -replace ":", "" }
Start-Transcript -Path "log/log-$timestamp.txt"

#---------------------------------------------------------------------------------
# Get install path and version
$installdir = Get-ItemPropertyValue -Path 'HKLM:\SOFTWARE\WOW6432Node\Bruker\Compass Server' -Name Path
"Server installed under $installdir"
$version = Get-ItemPropertyValue -Path 'HKLM:\SOFTWARE\WOW6432Node\Bruker\Compass Server' -Name Version
"Server version is $version"

# Check if installed version requires a patch
if (([System.Version]"$version" -ge [System.Version]"6.0") -Or ([System.Version]"$version" -lt [System.Version]"3.0"))
{
    if(([System.Version]"$version" -ge [System.Version]"6.1") -Or ((Get-ItemPropertyValue -Path 'HKLM:\SOFTWARE\WOW6432Node\Bruker\Compass Server' -Name InstallerScmRevision) -gt 29596))
	{
		"The installed version $version of the Compass Server System does not need to be patched."
		Exit
	}
}

#---------------------------------------------------------------------------------
# Stop server service
"Stopping server"
$serviceName = "bruker-compass-server"
$service = Get-Service -Name $serviceName
if (($service) -and ($service.Status -ne "stopped"))
{
    Stop-Service -Name $serviceName
}
elseif ($service)
{
    "Service '$serviceName' is not running."
}
else
{
    "Service '$serviceName' not found. Patch can't be applied."
	Exit
}
# Stop Karaf process if not already stopped by service
$karafPid = Get-Content "$installdir\karaf.pid"
if(Get-Process -Id $karafPid -ErrorAction Ignore) {
	"Killing process"
    Stop-Process -Id $karafPid -Force
    $timeSpan = New-TimeSpan -Seconds 30
    $endDate = (Get-Date) + $timeSpan
    while(Get-Process -Id $karafPid -ErrorAction Ignore)
    {
        if((Get-Date).CompareTo($endDate) -ge 0)
        {
            "Server did not stop within 30 s. Patch can't be applied."
            Exit
        }
        sleep 1
    }
}

#---------------------------------------------------------------------------------
# Disable lookup
$file = "$installdir\etc\system.properties"
$lineContent = "log4j2.formatMsgNoLookups=true"
$line = "`r`n$lineContent"

if ([System.Version]"$version" -gt [System.Version]"5.0")
{
	if((Get-Content $file | Select-String -pattern $lineContent).length -eq 0)
	{
		Add-Content -Path $file -Value "$line"
		if ($error.Count -ne 0)
		{
			"Disabling the lookup of message patterns was not successful!: $error.Count"
			"Continuing with patching the log4j library"
		}
		else
		{
			"Successfully disabled the lookup of message patterns!"
		}
	}
	else
	{
		"Lookup of message patterns already disabled"
	}
}

#---------------------------------------------------------------------------------
# Replace affected library with patched one
if ([System.Version]"$version" -gt [System.Version]"6.1")
{
	$log4j2Version = '2.0.10'
}
elseif ([System.Version]"$version" -gt [System.Version]"6.0")
{
	$log4j2Version = '2.0.8'
}
elseif ([System.Version]"$version" -gt [System.Version]"5.0")
{
	$log4j2Version = '1.11.4'
}
elseif ([System.Version]"$version" -ge [System.Version]"3.0")
{
	$log4j2Version = '1.10.1'
}

$rootPath = $PSScriptRoot
$jarFile = "pax-logging-log4j2-$log4j2Version.jar"
$sourceFolder = "$rootPath\patch-files"
$targetFolder = "$installdir\system\org\ops4j\pax\logging\pax-logging-log4j2\$log4j2Version"
"Patching file $targetFolder\$jarFile"
Copy-Item "$sourceFolder\$jarFile" -Destination "$targetFolder\$jarFile"

#---------------------------------------------------------------------------------
# Clean working directories
"Cleaning installation"
$emptyFolder = "EmptyFolder" + $timestamp
mkdir $emptyFolder > $null
$robocopyLog = 'log\robocopy-' + $timestamp + '.txt'
"Removing data folder"
robocopy $emptyFolder "$installdir\data" /mir /xd log > $robocopyLog
"Removing workspace folder"
$workspaceFolder = "$installdir\workspace"
robocopy $emptyFolder $workspaceFolder /mir >> $robocopyLog
Remove-Item -Path $workspaceFolder -Force -Recurse

#---------------------------------------------------------------------------------
# Remove dbscripts folder to remove unused log4j1 library
"Removing dbscripts folder"
$dbscriptsFolder = "$installdir\install\dbscripts"
robocopy $emptyFolder $dbscriptsFolder /mir >> $robocopyLog
Remove-Item -Path $dbscriptsFolder -Force -Recurse

Remove-Item -Path $emptyFolder -Force -Recurse

#---------------------------------------------------------------------------------
# Restart server
"Starting server"
Start-Service -Name "bruker-compass-server"

#---------------------------------------------------------------------------------
# Add patch info to Registry entries
Set-ItemProperty -Path 'HKLM:\SOFTWARE\WOW6432Node\Bruker\Compass Server' -Name Log4jPatchVersion -Value 1.7
$uninstallName = Get-ChildItem -Path "HKLM:\SOFTWARE\WOW6432Node\Microsoft\Windows\CurrentVersion\Uninstall" |
    Get-ItemProperty |
    Where-Object {$_.DisplayName -match "Bruker Compass Server \d[.]\d.*" } |
    Select-Object -ExpandProperty DisplayName
Set-ItemProperty -Path "HKLM:\SOFTWARE\WOW6432Node\Microsoft\Windows\CurrentVersion\Uninstall\$uninstallName" -Name Comments -Value "Patched with Log4j Patch Version 1.7"

Stop-Transcript

# SIG # Begin signature block
# MIImywYJKoZIhvcNAQcCoIImvDCCJrgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUa3OuPRpwvB3FJkR5B27/nHrr
# X7aggh+yMIIFgTCCBGmgAwIBAgIQOXJEOvkit1HX02wQ3TE1lTANBgkqhkiG9w0B
# AQwFADB7MQswCQYDVQQGEwJHQjEbMBkGA1UECAwSR3JlYXRlciBNYW5jaGVzdGVy
# MRAwDgYDVQQHDAdTYWxmb3JkMRowGAYDVQQKDBFDb21vZG8gQ0EgTGltaXRlZDEh
# MB8GA1UEAwwYQUFBIENlcnRpZmljYXRlIFNlcnZpY2VzMB4XDTE5MDMxMjAwMDAw
# MFoXDTI4MTIzMTIzNTk1OVowgYgxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpOZXcg
# SmVyc2V5MRQwEgYDVQQHEwtKZXJzZXkgQ2l0eTEeMBwGA1UEChMVVGhlIFVTRVJU
# UlVTVCBOZXR3b3JrMS4wLAYDVQQDEyVVU0VSVHJ1c3QgUlNBIENlcnRpZmljYXRp
# b24gQXV0aG9yaXR5MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAgBJl
# FzYOw9sIs9CsVw127c0n00ytUINh4qogTQktZAnczomfzD2p7PbPwdzx07HWezco
# EStH2jnGvDoZtF+mvX2do2NCtnbyqTsrkfjib9DsFiCQCT7i6HTJGLSR1GJk23+j
# BvGIGGqQIjy8/hPwhxR79uQfjtTkUcYRZ0YIUcuGFFQ/vDP+fmyc/xadGL1RjjWm
# p2bIcmfbIWax1Jt4A8BQOujM8Ny8nkz+rwWWNR9XWrf/zvk9tyy29lTdyOcSOk2u
# TIq3XJq0tyA9yn8iNK5+O2hmAUTnAU5GU5szYPeUvlM3kHND8zLDU+/bqv50TmnH
# a4xgk97Exwzf4TKuzJM7UXiVZ4vuPVb+DNBpDxsP8yUmazNt925H+nND5X4OpWax
# KXwyhGNVicQNwZNUMBkTrNN9N6frXTpsNVzbQdcS2qlJC9/YgIoJk2KOtWbPJYjN
# hLixP6Q5D9kCnusSTJV882sFqV4Wg8y4Z+LoE53MW4LTTLPtW//e5XOsIzstAL81
# VXQJSdhJWBp/kjbmUZIO8yZ9HE0XvMnsQybQv0FfQKlERPSZ51eHnlAfV1SoPv10
# Yy+xUGUJ5lhCLkMaTLTwJUdZ+gQek9QmRkpQgbLevni3/GcV4clXhB4PY9bpYrrW
# X1Uu6lzGKAgEJTm4Diup8kyXHAc/DVL17e8vgg8CAwEAAaOB8jCB7zAfBgNVHSME
# GDAWgBSgEQojPpbxB+zirynvgqV/0DCktDAdBgNVHQ4EFgQUU3m/WqorSs9UgOHY
# m8Cd8rIDZsswDgYDVR0PAQH/BAQDAgGGMA8GA1UdEwEB/wQFMAMBAf8wEQYDVR0g
# BAowCDAGBgRVHSAAMEMGA1UdHwQ8MDowOKA2oDSGMmh0dHA6Ly9jcmwuY29tb2Rv
# Y2EuY29tL0FBQUNlcnRpZmljYXRlU2VydmljZXMuY3JsMDQGCCsGAQUFBwEBBCgw
# JjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuY29tb2RvY2EuY29tMA0GCSqGSIb3
# DQEBDAUAA4IBAQAYh1HcdCE9nIrgJ7cz0C7M7PDmy14R3iJvm3WOnnL+5Nb+qh+c
# li3vA0p+rvSNb3I8QzvAP+u431yqqcau8vzY7qN7Q/aGNnwU4M309z/+3ri0ivCR
# lv79Q2R+/czSAaF9ffgZGclCKxO/WIu6pKJmBHaIkU4MiRTOok3JMrO66BQavHHx
# W/BBC5gACiIDEOUMsfnNkjcZ7Tvx5Dq2+UUTJnWvu6rvP3t3O9LEApE9GQDTF1w5
# 2z97GA1FzZOFli9d31kWTz9RvdVFGD/tSo7oBmF0Ixa1DVBzJ0RHfxBdiSprhTEU
# xOipakyAvGp4z7h/jnZymQyd/teRCBaho1+VMIIF9TCCA92gAwIBAgIQHaJIMG+b
# JhjQguCWfTPTajANBgkqhkiG9w0BAQwFADCBiDELMAkGA1UEBhMCVVMxEzARBgNV
# BAgTCk5ldyBKZXJzZXkxFDASBgNVBAcTC0plcnNleSBDaXR5MR4wHAYDVQQKExVU
# aGUgVVNFUlRSVVNUIE5ldHdvcmsxLjAsBgNVBAMTJVVTRVJUcnVzdCBSU0EgQ2Vy
# dGlmaWNhdGlvbiBBdXRob3JpdHkwHhcNMTgxMTAyMDAwMDAwWhcNMzAxMjMxMjM1
# OTU5WjB8MQswCQYDVQQGEwJHQjEbMBkGA1UECBMSR3JlYXRlciBNYW5jaGVzdGVy
# MRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxJDAi
# BgNVBAMTG1NlY3RpZ28gUlNBIENvZGUgU2lnbmluZyBDQTCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBAIYijTKFehifSfCWL2MIHi3cfJ8Uz+MmtiVmKUCG
# VEZ0MWLFEO2yhyemmcuVMMBW9aR1xqkOUGKlUZEQauBLYq798PgYrKf/7i4zIPoM
# GYmobHutAMNhodxpZW0fbieW15dRhqb0J+V8aouVHltg1X7XFpKcAC9o95ftanK+
# ODtj3o+/bkxBXRIgCFnoOc2P0tbPBrRXBbZOoT5Xax+YvMRi1hsLjcdmG0qfnYHE
# ckC14l/vC0X/o84Xpi1VsLewvFRqnbyNVlPG8Lp5UEks9wO5/i9lNfIi6iwHr0bZ
# +UYc3Ix8cSjz/qfGFN1VkW6KEQ3fBiSVfQ+noXw62oY1YdMCAwEAAaOCAWQwggFg
# MB8GA1UdIwQYMBaAFFN5v1qqK0rPVIDh2JvAnfKyA2bLMB0GA1UdDgQWBBQO4Tqo
# Uzox1Yq+wbutZxoDha00DjAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB
# /wIBADAdBgNVHSUEFjAUBggrBgEFBQcDAwYIKwYBBQUHAwgwEQYDVR0gBAowCDAG
# BgRVHSAAMFAGA1UdHwRJMEcwRaBDoEGGP2h0dHA6Ly9jcmwudXNlcnRydXN0LmNv
# bS9VU0VSVHJ1c3RSU0FDZXJ0aWZpY2F0aW9uQXV0aG9yaXR5LmNybDB2BggrBgEF
# BQcBAQRqMGgwPwYIKwYBBQUHMAKGM2h0dHA6Ly9jcnQudXNlcnRydXN0LmNvbS9V
# U0VSVHJ1c3RSU0FBZGRUcnVzdENBLmNydDAlBggrBgEFBQcwAYYZaHR0cDovL29j
# c3AudXNlcnRydXN0LmNvbTANBgkqhkiG9w0BAQwFAAOCAgEATWNQ7Uc0SmGk295q
# Koyb8QAAHh1iezrXMsL2s+Bjs/thAIiaG20QBwRPvrjqiXgi6w9G7PNGXkBGiRL0
# C3danCpBOvzW9Ovn9xWVM8Ohgyi33i/klPeFM4MtSkBIv5rCT0qxjyT0s4E307dk
# sKYjalloUkJf/wTr4XRleQj1qZPea3FAmZa6ePG5yOLDCBaxq2NayBWAbXReSnV+
# pbjDbLXP30p5h1zHQE1jNfYw08+1Cg4LBH+gS667o6XQhACTPlNdNKUANWlsvp8g
# JRANGftQkGG+OY96jk32nw4e/gdREmaDJhlIlc5KycF/8zoFm/lv34h/wCOe0h5D
# ekUxwZxNqfBZslkZ6GqNKQQCd3xLS81wvjqyVVp4Pry7bwMQJXcVNIr5NsxDkuS6
# T/FikyglVyn7URnHoSVAaoRXxrKdsbwcCtp8Z359LukoTBh+xHsxQXGaSynsCz1X
# UNLK3f2eBVHlRHjdAd6xdZgNVCT98E7j4viDvXK6yz067vBeF5Jobchh+abxKgoL
# pbn0nu6YMgWFnuv5gynTxix9vTp3Los3QqBqgu07SqqUEKThDfgXxbZaeTMYkuO1
# dfih6Y4KJR7kHvGfWocj/5+kUZ77OYARzdu1xKeogG/lU9Tg46LC0lsa+jImLWpX
# cBw8pFguo/NbSwfcMlnzh6cabVgwggY1MIIFHaADAgECAhEApsOP1F1Sucttusrp
# 9X63yjANBgkqhkiG9w0BAQsFADB8MQswCQYDVQQGEwJHQjEbMBkGA1UECBMSR3Jl
# YXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQKEw9TZWN0
# aWdvIExpbWl0ZWQxJDAiBgNVBAMTG1NlY3RpZ28gUlNBIENvZGUgU2lnbmluZyBD
# QTAeFw0yMDA1MDQwMDAwMDBaFw0yMzA1MDQyMzU5NTlaMIGbMQswCQYDVQQGEwJE
# RTEOMAwGA1UEEQwFMjgzNTkxDzANBgNVBAgMBkJyZW1lbjEPMA0GA1UEBwwGQnJl
# bWVuMRwwGgYDVQQJDBNGYWhyZW5oZWl0c3RyYcOfZSA0MR0wGwYDVQQKDBRCcnVr
# ZXIgRGFsdG9uaWsgR21iSDEdMBsGA1UEAwwUQnJ1a2VyIERhbHRvbmlrIEdtYkgw
# ggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQCza7uXxucE1MHMGt6UQxlC
# 27uEP7TyZnKZesYcXUGIn0aenv6tVm16JegblW+GUiBLvW+ogH2j1PipEAon0Dc6
# Ejm11BYy0pLSKBIavbBV32nMx1HQSBhIDP8w/YqfrOr8F1kxvc8LRNaiDM8IYbUa
# IFGiC9UgS284WMnrMKoGIgBLXdr+T9c/eQARZIhcqKizmlI6CSEtHk+Navff2isw
# J9jRgkJM4V7EXK3lc8I1zIJoFBV10dvODcsf+TX0Fki6yKqmXcYYfmwaeeGv6UcD
# 0Vk2/HQzAa83vPV6FfXuR2+ooBiBb9/4wqmkhzssSc2qwJFQxygB6UQNOYeSuOMg
# YoqrUSJMQvrEgyLLR5fv9W6Z1jonyxmcgBo1KOm+bQGGi6EA7bnaEd9HqXXwm+JN
# iEr1WUgymPryr4N71OLxpvaBEGLzJzWvsmXP4UU7h7prC913cRMnovTccVdy2v2l
# 2Vll2qJ1TAP3s6ol4cQWDd2W6qQE+tDzpIYnK3L/FLctIox4eRgynP8Eah7Q3NtK
# RLMLytj+MjCXJ6yG/+uO+mSLt+2Wbth4A8RLUvQLVrlYlZbwcX7mYVp04t9xxHw/
# kQ6XP0hSdqtL4Jnej1DUnX8Q4Nnx6rqzInaWHwCBmBg+E4eYBFKdw3bW/+3U0TXa
# fTj5jmKI+AfvKS1oQIvCjQIDAQABo4IBkDCCAYwwHwYDVR0jBBgwFoAUDuE6qFM6
# MdWKvsG7rWcaA4WtNA4wHQYDVR0OBBYEFAn3pDU0ccnZModfIisCwzJRTxYEMA4G
# A1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBMGA1UdJQQMMAoGCCsGAQUFBwMD
# MBEGCWCGSAGG+EIBAQQEAwIEEDBKBgNVHSAEQzBBMDUGDCsGAQQBsjEBAgEDAjAl
# MCMGCCsGAQUFBwIBFhdodHRwczovL3NlY3RpZ28uY29tL0NQUzAIBgZngQwBBAEw
# QwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5zZWN0aWdvLmNvbS9TZWN0aWdv
# UlNBQ29kZVNpZ25pbmdDQS5jcmwwcwYIKwYBBQUHAQEEZzBlMD4GCCsGAQUFBzAC
# hjJodHRwOi8vY3J0LnNlY3RpZ28uY29tL1NlY3RpZ29SU0FDb2RlU2lnbmluZ0NB
# LmNydDAjBggrBgEFBQcwAYYXaHR0cDovL29jc3Auc2VjdGlnby5jb20wDQYJKoZI
# hvcNAQELBQADggEBAB8EKzep634Zg97JLZLT72+cThTsLyy+EOaQzn4cqfIL1l5T
# awNG6/ulTezTjd4zKkg78M9aPDzui2saNbQ+lT7spgm7r6vjo9IT+i83UuYuE4dm
# ieW3eaQJW9K9XTtRy62wZdHGGl6ewwAiGG73/YsPeiN8rMpKO4CRAMe5Rwj8NuTw
# QsgPTUYa3j9lfldYKGFOjXbQzMcmc3eFaT2L//6LGYKVqmfkvHSX8g4wKUmNkcIL
# hRqgYcEVBdUlhjzwTy4KiTOoJ5ii24/jUaGjuJmVTBgtisB1030+RS4UrLu+HIgd
# eSiE0uMp6xpzgLsxsRDXoEMfZLDr2b6kojoRbG8wggbsMIIE1KADAgECAhAwD2+s
# 3WaYdHypRjaneC25MA0GCSqGSIb3DQEBDAUAMIGIMQswCQYDVQQGEwJVUzETMBEG
# A1UECBMKTmV3IEplcnNleTEUMBIGA1UEBxMLSmVyc2V5IENpdHkxHjAcBgNVBAoT
# FVRoZSBVU0VSVFJVU1QgTmV0d29yazEuMCwGA1UEAxMlVVNFUlRydXN0IFJTQSBD
# ZXJ0aWZpY2F0aW9uIEF1dGhvcml0eTAeFw0xOTA1MDIwMDAwMDBaFw0zODAxMTgy
# MzU5NTlaMH0xCzAJBgNVBAYTAkdCMRswGQYDVQQIExJHcmVhdGVyIE1hbmNoZXN0
# ZXIxEDAOBgNVBAcTB1NhbGZvcmQxGDAWBgNVBAoTD1NlY3RpZ28gTGltaXRlZDEl
# MCMGA1UEAxMcU2VjdGlnbyBSU0EgVGltZSBTdGFtcGluZyBDQTCCAiIwDQYJKoZI
# hvcNAQEBBQADggIPADCCAgoCggIBAMgbAa/ZLH6ImX0BmD8gkL2cgCFUk7nPoD5T
# 77NawHbWGgSlzkeDtevEzEk0y/NFZbn5p2QWJgn71TJSeS7JY8ITm7aGPwEFkmZv
# IavVcRB5h/RGKs3EWsnb111JTXJWD9zJ41OYOioe/M5YSdO/8zm7uaQjQqzQFcN/
# nqJc1zjxFrJw06PE37PFcqwuCnf8DZRSt/wflXMkPQEovA8NT7ORAY5unSd1VdEX
# OzQhe5cBlK9/gM/REQpXhMl/VuC9RpyCvpSdv7QgsGB+uE31DT/b0OqFjIpWcdEt
# lEzIjDzTFKKcvSb/01Mgx2Bpm1gKVPQF5/0xrPnIhRfHuCkZpCkvRuPd25Ffnz82
# Pg4wZytGtzWvlr7aTGDMqLufDRTUGMQwmHSCIc9iVrUhcxIe/arKCFiHd6QV6xlV
# /9A5VC0m7kUaOm/N14Tw1/AoxU9kgwLU++Le8bwCKPRt2ieKBtKWh97oaw7wW33p
# dmmTIBxKlyx3GSuTlZicl57rjsF4VsZEJd8GEpoGLZ8DXv2DolNnyrH6jaFkyYiS
# WcuoRsDJ8qb/fVfbEnb6ikEk1Bv8cqUUotStQxykSYtBORQDHin6G6UirqXDTYLQ
# jdprt9v3GEBXc/Bxo/tKfUU2wfeNgvq5yQ1TgH36tjlYMu9vGFCJ10+dM70atZ2h
# 3pVBeqeDAgMBAAGjggFaMIIBVjAfBgNVHSMEGDAWgBRTeb9aqitKz1SA4dibwJ3y
# sgNmyzAdBgNVHQ4EFgQUGqH4YRkgD8NBd0UojtE1XwYSBFUwDgYDVR0PAQH/BAQD
# AgGGMBIGA1UdEwEB/wQIMAYBAf8CAQAwEwYDVR0lBAwwCgYIKwYBBQUHAwgwEQYD
# VR0gBAowCDAGBgRVHSAAMFAGA1UdHwRJMEcwRaBDoEGGP2h0dHA6Ly9jcmwudXNl
# cnRydXN0LmNvbS9VU0VSVHJ1c3RSU0FDZXJ0aWZpY2F0aW9uQXV0aG9yaXR5LmNy
# bDB2BggrBgEFBQcBAQRqMGgwPwYIKwYBBQUHMAKGM2h0dHA6Ly9jcnQudXNlcnRy
# dXN0LmNvbS9VU0VSVHJ1c3RSU0FBZGRUcnVzdENBLmNydDAlBggrBgEFBQcwAYYZ
# aHR0cDovL29jc3AudXNlcnRydXN0LmNvbTANBgkqhkiG9w0BAQwFAAOCAgEAbVSB
# pTNdFuG1U4GRdd8DejILLSWEEbKw2yp9KgX1vDsn9FqguUlZkClsYcu1UNviffmf
# AO9Aw63T4uRW+VhBz/FC5RB9/7B0H4/GXAn5M17qoBwmWFzztBEP1dXD4rzVWHi/
# SHbhRGdtj7BDEA+N5Pk4Yr8TAcWFo0zFzLJTMJWk1vSWVgi4zVx/AZa+clJqO0I3
# fBZ4OZOTlJux3LJtQW1nzclvkD1/RXLBGyPWwlWEZuSzxWYG9vPWS16toytCiiGS
# /qhvWiVwYoFzY16gu9jc10rTPa+DBjgSHSSHLeT8AtY+dwS8BDa153fLnC6NIxi5
# o8JHHfBd1qFzVwVomqfJN2Udvuq82EKDQwWli6YJ/9GhlKZOqj0J9QVst9JkWtgq
# IsJLnfE5XkzeSD2bNJaaCV+O/fexUpHOP4n2HKG1qXUfcb9bQ11lPVCBbqvw0NP8
# srMftpmWJvQ8eYtcZMzN7iea5aDADHKHwW5NWtMe6vBE5jJvHOsXTpTDeGUgOw9B
# qh/poUGd/rG4oGUqNODeqPk85sEwu8CgYyz8XBYAqNDEf+oRnR4GxqZtMl20OAkr
# SQeq/eww2vGnL8+3/frQo4TZJ577AWZ3uVYQ4SBuxq6x+ba6yDVdM3aO8XwgDCp3
# rrWiAoa6Ke60WgCxjKvj+QrJVF3UuWp0nr1IrpgwggcHMIIE76ADAgECAhEAjHeg
# AI/00bDGPZ86SIONazANBgkqhkiG9w0BAQwFADB9MQswCQYDVQQGEwJHQjEbMBkG
# A1UECBMSR3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUg
# U3RhbXBpbmcgQ0EwHhcNMjAxMDIzMDAwMDAwWhcNMzIwMTIyMjM1OTU5WjCBhDEL
# MAkGA1UEBhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3RlcjEQMA4GA1UE
# BxMHU2FsZm9yZDEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMSwwKgYDVQQDDCNT
# ZWN0aWdvIFJTQSBUaW1lIFN0YW1waW5nIFNpZ25lciAjMjCCAiIwDQYJKoZIhvcN
# AQEBBQADggIPADCCAgoCggIBAJGHSyyLwfEeoJ7TB8YBylKwvnl5XQlmBi0vNX27
# wPsn2kJqWRslTOrvQNaafjLIaoF9tFw+VhCBNToiNoz7+CAph6x00BtivD9khwJf
# 78WA7wYc3F5Ok4e4mt5MB06FzHDFDXvsw9njl+nLGdtWRWzuSyBsyT5s/fCb8Sj4
# kZmq/FrBmoIgOrfv59a4JUnCORuHgTnLw7c6zZ9QBB8amaSAAk0dBahV021SgIPm
# bkilX8GJWGCK7/GszYdjGI50y4SHQWljgbz2H6p818FBzq2rdosggNQtlQeNx/UL
# Fx6a5daZaVHHTqadKW/neZMNMmNTrszGKYogwWDG8gIsxPnIIt/5J4Khg1HCvMmC
# GiGEspe81K9EHJaCIpUqhVSu8f0+SXR0/I6uP6Vy9MNaAapQpYt2lRtm6+/a35Qu
# 2RrrTCd9TAX3+CNdxFfIJgV6/IEjX1QJOCpi1arK3+3PU6sf9kSc1ZlZxVZkW/eO
# Ug9m/Jg/RAYTZG7p4RVgUKWx7M+46MkLvsWE990Kndq8KWw9Vu2/eGe2W8heFBy5
# r4Qtd6L3OZU3b05/HMY8BNYxxX7vPehRfnGtJHQbLNz5fKrvwnZJaGLVi/UD3759
# jg82dUZbk3bEg+6CviyuNxLxvFbD5K1Dw7dmll6UMvqg9quJUPrOoPMIgRrRRKfM
# 97gxAgMBAAGjggF4MIIBdDAfBgNVHSMEGDAWgBQaofhhGSAPw0F3RSiO0TVfBhIE
# VTAdBgNVHQ4EFgQUaXU3e7udNUJOv1fTmtufAdGu3tAwDgYDVR0PAQH/BAQDAgbA
# MAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwQAYDVR0gBDkw
# NzA1BgwrBgEEAbIxAQIBAwgwJTAjBggrBgEFBQcCARYXaHR0cHM6Ly9zZWN0aWdv
# LmNvbS9DUFMwRAYDVR0fBD0wOzA5oDegNYYzaHR0cDovL2NybC5zZWN0aWdvLmNv
# bS9TZWN0aWdvUlNBVGltZVN0YW1waW5nQ0EuY3JsMHQGCCsGAQUFBwEBBGgwZjA/
# BggrBgEFBQcwAoYzaHR0cDovL2NydC5zZWN0aWdvLmNvbS9TZWN0aWdvUlNBVGlt
# ZVN0YW1waW5nQ0EuY3J0MCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdv
# LmNvbTANBgkqhkiG9w0BAQwFAAOCAgEASgN4kEIz7Hsagwk2M5hVu51ABjBrRWrx
# lA4ZUP9bJV474TnEW7rplZA3N73f+2Ts5YK3lcxXVXBLTvSoh90ihaZXu7ghJ9Sg
# KjGUigchnoq9pxr1AhXLRFCZjOw+ugN3poICkMIuk6m+ITR1Y7ngLQ/PATfLjaL6
# uFqarqF6nhOTGVWPCZAu3+qIFxbradbhJb1FCJeA11QgKE/Ke7OzpdIAsGA0ZcTj
# xcOl5LqFqnpp23WkPnlomjaLQ6421GFyPA6FYg2gXnDbZC8Bx8GhxySUo7I8brJe
# otD6qNG4JRwW5sDVf2gaxGUpNSotiLzqrnTWgufAiLjhT3jwXMrAQFzCn9UyHCza
# PKw29wZSmqNAMBewKRaZyaq3iEn36AslM7U/ba+fXwpW3xKxw+7OkXfoIBPpXCTH
# 6kQLSuYThBxN6w21uIagMKeLoZ+0LMzAFiPJkeVCA0uAzuRN5ioBPsBehaAkoRdA
# 1dvb55gQpPHqGRuAVPpHieiYgal1wA7f0GiUeaGgno62t0Jmy9nZay9N2N4+Mh4g
# 5OycTUKNncczmYI3RNQmKSZAjngvue76L/Hxj/5QuHjdFJbeHA5wsCqFarFsaOkq
# 5BArbiH903ydN+QqBtbD8ddo408HeYEIE/6yZF7psTzm0Hgjsgks4iZivzupl1HM
# x0QygbKvz98xggaDMIIGfwIBATCBkTB8MQswCQYDVQQGEwJHQjEbMBkGA1UECBMS
# R3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQKEw9T
# ZWN0aWdvIExpbWl0ZWQxJDAiBgNVBAMTG1NlY3RpZ28gUlNBIENvZGUgU2lnbmlu
# ZyBDQQIRAKbDj9RdUrnLbbrK6fV+t8owCQYFKw4DAhoFAKB4MBgGCisGAQQBgjcC
# AQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYB
# BAGCNwIBCzEOMAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFPEwgbEC9iVo
# Pwx1ggGEk54AcR90MA0GCSqGSIb3DQEBAQUABIICABGwaagWNq+2g3BmdvJPI+Eg
# J+Xlw/w0IML3v5bEzUj4CA+Vbh/rrPcLNST8H6GQj5MJ7pLm+a0TSYHBeSInuLE0
# CO6VQlcTze1qfZ1V6xZz4sfJmi6ql0WBN67k2JmdLcl1zdHBqNjo128Ucf8DAibe
# 2orh73jkAi6Pt/7GJvb44K0rrX7ahrnLrv0DNLW51Bh5SD1nn7re3KJU5tRgYoYl
# zICwnnLdyCunRLhIYnSdClBAwUzsH7R2DKQXS3GL5ABEq2QVI7pgQuTa/RsLJ7Hi
# OeewA5mi5pg1d4vtpm1NwBXiz5joBJHJT2IogZBRWJfzpfG6h6mQ+TrXOIjrivI2
# UrURtWe662kRcFROKxBsz+bMcs4e4QY2Nmscg+419BdA+HTigymnuDAwhxHBRoGh
# RbfusobtPhz5ICLy8xb7882+xe67CtOT4MkokbFGggeMyfQ1O8OSJrsxkMCr9pqG
# WDFpeB0M8gaMWBneiZZtoM3mEEyvRwfDwoEyKMFmJpppwP8KdCwUn2FQBdfEbwnz
# 3hjgk3Apf5lnHM7fdYT7C47UiV9NbJ+3yRqPpj000h8dSvhBX08hs1Adk02S3Mb6
# QGe4kqaEOAmO+SgsIa03eR1sRnluIhas382Sa1UuL22f9te/0R//0f4plcW1q1eJ
# +rqe6SDZ6gml4sv37rK/oYIDTDCCA0gGCSqGSIb3DQEJBjGCAzkwggM1AgEBMIGS
# MH0xCzAJBgNVBAYTAkdCMRswGQYDVQQIExJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAO
# BgNVBAcTB1NhbGZvcmQxGDAWBgNVBAoTD1NlY3RpZ28gTGltaXRlZDElMCMGA1UE
# AxMcU2VjdGlnbyBSU0EgVGltZSBTdGFtcGluZyBDQQIRAIx3oACP9NGwxj2fOkiD
# jWswDQYJYIZIAWUDBAICBQCgeTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwG
# CSqGSIb3DQEJBTEPFw0yMjAyMDMxNDAxMDFaMD8GCSqGSIb3DQEJBDEyBDBa8unS
# oBKeq1zk+sTF112GwRTuN3yMSvYn7omiPv++PhGGBfwevhnbV7lXPgqiPWQwDQYJ
# KoZIhvcNAQEBBQAEggIAhZYYZxae2RmX/IP8Vu89QG7xkhICglqBrwYPKk9FKbAz
# c6iaV1bujdVYZ4JLNcgoN84q27lAwLP2byGznqa2NFm8NAdapNXrmCd+lWjNRmB0
# os+wVqTWaGkITRP6eEJQzWvqJxpnzkGSluEQrSpHJC0dLjDFl+0RGlrxsf4CywnD
# DxgBws9wDeTSlNmXdWRXHNKzZw2GvRQXFyNov+cahofjajITx6Z0wDvs7VsM8+IJ
# v2xtQOH7Xezo/sMcxxpLECpYOF/8H0Ou6wEoeFI4Q0GDcpz3uJCtHpZ0qA8ywDHL
# jONVP4909s4ZE0KEQR61Dga4eVOczhsiNLju3tAP5Ak2p3A6weBiHdeIsZZ2wty9
# eV4dukYYd3IaRVj5Y3/3bzvXmTh9QlxRXbIKxSoJYNI8n5JuXcoPMF4cnMJDCigW
# 1YxksZdk/JVia3HtAFZL8YCnIzolt5Jq5s+0/T8SHetaDUgJRD1IQasoeWxwUGxk
# eAQfTe+woXHqjNDL3Y/A3i7PNJOlq1WGgL4OPWWMpfOPBhNWnnsgCHuO5TGJaGUH
# CNYFpumpa4d1o60rNVK13sfrG3R7c+bBpsIpuNOX+dOfkI5jy5/I8YoDiP1PyNa8
# qM0139I9oFezDc456hBdAEAbXUpZgl+1iRfPzPJLw8saDUi2XrP51S1lAWiFqsg=
# SIG # End signature block
